/*
Copyright 2025 BewitchedFlora
SPDX-License-Identifier: Apache-2.0
*/

#include "/lib/code/options.glsl"

uniform float viewWidth;
uniform float viewHeight;
uniform sampler2D gtexture;
uniform sampler2D noise;

in vec2 texCoord;

layout(location = 0) out vec4 pixelColor;

const int BLOOM_KERNEL_SIZE = 4;
const float BLOOM_KERNEL[BLOOM_KERNEL_SIZE] = float[](0.227027, 0.1945946, 0.1216216, 0.054054);

vec3 get_bloom_color(vec2 coord) {
    vec4 texColor = texture(gtexture, coord);
    float brightness = dot(texColor.rgb, vec3(0.2126, 0.7152, 0.0722));
    return (brightness > 0.95) ? texColor.rgb : vec3(0.0);
}

void main() {
	vec3 result = get_bloom_color(texCoord) * BLOOM_KERNEL[0];
	vec2 texOffset = 5 / vec2(viewWidth, viewHeight);
	
	for(int i = 1; i < BLOOM_KERNEL_SIZE; ++i) {
		result += get_bloom_color(texCoord + vec2(texOffset.x * i, 0.0)).rgb * BLOOM_KERNEL[i];
		result += get_bloom_color(texCoord - vec2(texOffset.x * i, 0.0)).rgb * BLOOM_KERNEL[i];
		result += get_bloom_color(texCoord + vec2(0.0, texOffset.x * i)).rgb * BLOOM_KERNEL[i];
		result += get_bloom_color(texCoord - vec2(0.0, texOffset.x * i)).rgb * BLOOM_KERNEL[i];
     }
	
	vec4 texColor = texture(gtexture, texCoord);
	
	pixelColor = vec4(texColor.rgb + result / BLOOM_KERNEL_SIZE, 1.0);
}
