/*
Copyright 2025 BewitchedFlora
SPDX-License-Identifier: Apache-2.0
*/

#include "/lib/code/options.glsl"

attribute vec4 mc_Entity;

out vec2 lightCoord;
out vec2 texCoord;
out vec3 normal;
out vec3 vertexPosition;
out float reflectiveness;
out float vertexDistance;
out vec4 vertexColor;
out float cloudiness;

uniform vec3 cameraPosition;
uniform int blockEntityId;
uniform float frameTimeCounter;
uniform int isEyeInWater;
uniform vec4 entityColor;

void main() {
	int entityID = int(mc_Entity.x + 0.5);
	vec4 position = gl_Vertex;

	// Set the material properties based on entity ID.
	reflectiveness = step(10000, entityID);
	cloudiness = step(10001, entityID);
	vec4 vertexColorModifier = 1.0 + step(0.5, float(entityID - 10002)) * vec4(4.0, -0.05, -0.05, 1.0);
	vec4 finalColor = gl_Color*vertexColorModifier;

	// Move water and lava around.
	#ifdef SHADER_TERRAIN
		#ifdef WATER_WAVES
			position.y += step(10001, entityID) * (sin(frameTimeCounter * 1.0 + (position.xyz + cameraPosition).z) / 30.0);
		#endif
	#endif

	#ifdef DIMENSION_NETHER
		#ifdef NETHER_BREATHING
			position.xyz += (1-reflectiveness) * (sin(frameTimeCounter * 1.0 + (position.xyz + cameraPosition).z) / 30.0) * position.xyz;
		#endif
	#else
		#ifdef WATER_CAUSTICS
			position.xyz += isEyeInWater * (sin(frameTimeCounter * 1.0 + (position.xyz + cameraPosition).z) / 30.0) * position.xyz;
		#endif
	#endif

	// Output variables.
	lightCoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
	normal = gl_Normal;
	texCoord = gl_MultiTexCoord0.xy;
	vertexColor = mix(finalColor + entityColor * 1.2, finalColor, 1.0 - entityColor.r/1.5);
	vertexDistance = length((gl_ModelViewMatrix * position).xyz);
	vertexPosition = position.xyz;
	gl_Position = gl_ModelViewProjectionMatrix * position;
}
